

import java.util.HashMap;
import java.util.Map;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.objects.P2Object;

/**
 * Implements a hash or dictionary. This object holds name/object pairs. Objects
 * may be added and removed.
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class SampleObject implements ManagedObject {

  private Map<String, P2Object> data;
  
  public static SampleObject newOne(String type) {
    return new SampleObject();
  }

  /**
   * Creates an empty hash
   */
  @Ponder2op("create")
  SampleObject() {
    data = new HashMap<String, P2Object>();
  }

  /**
   * Creates a hash with a particular minimum size of anInteger
   * 
   * @param anInteger
   *          the size with which to create the hash
   */
  @Ponder2op("size:")
  SampleObject(int anInteger) {
    data = new HashMap<String, P2Object>(anInteger);
    P2Object.create(4);
  }

  /**
   * add an aManagedObject to the hash with aName
   * 
   * @param aName
   *          the name of the object to be added
   * @param aManagedObject
   *          the object to be added
   * @return the object added
   */
  @Ponder2op("at:put:")
  P2Object p2_operation_at_put(String aName, P2Object aManagedObject) {
    data.put(aName, aManagedObject);
    return aManagedObject;
  }

  /**
   * retrieves an object using aName
   * 
   * @param aNname
   *          the name of the object to be found
   * @return the named object
   */
  @Ponder2op("at:")
  P2Object p2_operation_at(String aNname) {
    return data.get(aNname);
  }

  /**
   * removes the object called aName from the hash
   * 
   * @param aName
   *          the name of the object to be removed
   * @return the object removed
   */
  @Ponder2op("remove")
  P2Object p2_operation_remove(String aName) {
    return data.remove(aName);
  }

  /**
   * get the number of objects in the hash
   * 
   * @return the number of objects in the hash
   */
  @Ponder2op("size")
  int p2_operation_size() {
    return data.size();
  }

}
