package net.ponder2.bsn;

import java.awt.BorderLayout;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Jul 2, 2006
 *
 * $Log:$
 */

/**
 * TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class BSN extends JFrame {

  public enum BSNType {
    OXY, BP, HEART, TEMP, GLUCOSE, IPUMP
  };

  private static final long serialVersionUID = 1L;
  private JPanel jContentPane = null;
  private SliderValue value = null;
  private SliderValue rate = null;
  private JLabel icon = null;
  private JPanel jPanel = null;

  // private String iconName = "heart";
  private ImageIcon picture[];
  BSNReceiverInterface receiver;
  int newValue;

  private Thread thread = null;
  private Thread hbthread = null;
  boolean running = true;
  int pic = 0;
  int delay = 1000;

  String bsnType; // @jve:decl-index=0:
  String bsnName;
  private String patientName;
  private BSNWindowReceiver bsnWindowReceiver;

  static Dictionary<Integer, JLabel> rateLabels;
  static {
    rateLabels = new Hashtable<Integer, JLabel>();
    rateLabels.put(0, new JLabel("0.1"));
    rateLabels.put(1, new JLabel("0.5"));
    rateLabels.put(2, new JLabel("1"));
    rateLabels.put(3, new JLabel("2"));
    rateLabels.put(4, new JLabel("5"));
    rateLabels.put(5, new JLabel("10"));
  }

  /**
   * This is the default constructor
   */
  public BSN() {
    super();
    picture = new ImageIcon[2];
    newValue = 0;
    initialize();
    rate.setLabelTable(rateLabels);
    rate.setTitle("Scan Rate");
    rate.setUnits("sec(s)");
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {
    this.setSize(378, 143);
    this.setContentPane(getJContentPane());
    this.setTitle("BSNInterface");
    this.addWindowListener(new java.awt.event.WindowAdapter() {

      @Override
      public void windowActivated(WindowEvent e) {
        System.out.println("WindowListener method called: windowActivated.");
        if (BSN.this.isVisible() && !running) {
          running = true;
          startHeartbeat();
          BSN.this.setRate(delay);
          rate.setValue(delay / 1000.0);
        }
      }

    });
  }

  public void initialise(BSNType bsnType, String patientName, String name) {
    this.bsnType = bsnType.toString();
    this.patientName = patientName;
    this.bsnName = name;
    try {
      this.bsnWindowReceiver = new BSNWindowReceiver();
      Naming.rebind(patientName + "_" + name, bsnWindowReceiver);
    }
    catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    catch (MalformedURLException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    switch (bsnType) {
      case OXY:
        init("Oxygen Monitor - " + patientName + " " + name, "% Saturation", 0, 100, 95, 5000);
        initPicture("oxy");
        break;
      case BP:
        init("Blood Pressure Monitor - " + patientName + " " + name, "Blood Pressure", 30, 210, 95,
            2000);
        initPicture("bp");
        break;
      case TEMP:
        init("Temperature Monitor - " + patientName + " " + name, "Celcius", 30, 40, 36, 2000);
        initPicture("temp");
        break;
      case GLUCOSE:
        init("Glucose Level Monitor - " + patientName + " " + name, "Glucose mg/dl", 30, 250, 120,
            5000);
        initPicture("ecg");
        break;
      case HEART:
        init("Heart Rate Monitor - " + patientName + " " + name, "Heart Rate", 30, 210, 90, 2000);
        initPicture("heart");
        break;
      case IPUMP:
        System.err.println("IPUMP should not be referenced in switch statement");
        break;
    }
    setVisible(true);
  }

  /**
   * This method initializes jContentPane
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      icon = new JLabel();
      icon.setText(" ");
      jContentPane = new JPanel();
      jContentPane.setLayout(new BorderLayout());
      jContentPane.add(icon, BorderLayout.WEST);
      jContentPane.add(getJPanel(), BorderLayout.CENTER);
    }
    return jContentPane;
  }

  /**
   * This method initializes value
   * 
   * @return SliderValue
   */
  private SliderValue getValue() {
    if (value == null) {
      value = new SliderValue();
      value.add(getRate(), BorderLayout.SOUTH);
      value.addPropertyChangeListener("value", new java.beans.PropertyChangeListener() {

        public void propertyChange(java.beans.PropertyChangeEvent e) {
          System.out.println("value propertyChange(" + e.getNewValue() + ")"); // TODO
          newValue = Integer.parseInt((String)e.getNewValue());
        }
      });
    }
    return value;
  }

  /**
   * This method initializes rate
   * 
   * @return SliderValue
   */
  private SliderValue getRate() {
    if (rate == null) {
      rate = new SliderValue();
      rate.addPropertyChangeListener("value", new java.beans.PropertyChangeListener() {

        public void propertyChange(java.beans.PropertyChangeEvent e) {
          System.out.println("rate propertyChange(" + e.getNewValue() + ")");
          String srate = (String)e.getNewValue();
          double dvalue = Double.parseDouble(srate);
          dvalue *= 1000;
          setRate((int)dvalue);
        }
      });
    }
    return rate;
  }

  /**
   * This method initializes jPanel
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getJPanel() {
    if (jPanel == null) {
      jPanel = new JPanel();
      jPanel.setLayout(new BorderLayout());
      jPanel.add(getValue(), BorderLayout.CENTER);
    }
    return jPanel;
  }

  public void init(String title, String label, int min, int max, int initialValue, int initialRate) {
    setTitle(title);
    value.setTitle(label);
    value.setMinMax(min, max);
    value.setValue(initialValue);
    setRate(initialRate);
    rate.setValue(initialRate / 1000.0);
    startHeartbeat();
  }

  public void initPicture(String name) {
    picture[0] = new ImageIcon(this.getClass().getResource("/resource/" + name + "On.jpg"));
    picture[1] = new ImageIcon(this.getClass().getResource("/resource/" + name + "Off.jpg"));
    icon.setIcon(picture[0]);
  }

  private void checkVisible() {
    if (!isVisible()) {
      running = false;
      if (thread != null)
        thread.interrupt();
    }
  }

  private void setRate(int newDelay) {
    int oldDelay = delay;
    delay = newDelay;
    if (thread != null) {
      if (oldDelay > newDelay)
        thread.interrupt();
    }
    else {
      thread = new Thread() {

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Thread#run()
         */
        @Override
        public void run() {
          while (running) {
            try {
              // System.out.println("rate)");
              if (receiver != null) {
                try {
                  receiver.value(bsnType, bsnName, newValue, delay);
                  pic ^= 1;
                  icon.setIcon(picture[pic]);
                }
                catch (RemoteException e) {
                  receiver = null;
                }
              }
              Thread.sleep(delay);
              checkVisible();
            }
            catch (InterruptedException e) {
            }
          }
          thread = null;
          System.out.println(bsnName + " stopping");
        }

      };
      thread.start();
    }
  }

  private void startHeartbeat() {
    if (hbthread == null) {
      hbthread = new Thread() {

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Thread#run()
         */
        @Override
        public void run() {
          while (running) {
            try {
              // System.err.println("hbeat");
              if (receiver == null) {
                try {
                  // System.out.println("lookup");
                  receiver = (BSNReceiverInterface)Naming.lookup(patientName + "_" + "BSNBus");
                  // System.out.println("lookup done");
                }
                catch (MalformedURLException e) {
                  // TODO Auto-generated catch block
                  e.printStackTrace();
                }
                catch (RemoteException e) {
                  // TODO Auto-generated catch block
                  e.printStackTrace();
                }
                catch (NotBoundException e) {
                  // System.out.println("not bound");
                }
                // System.out.println("lookup over");

              }
              if (receiver != null) {
                try {
                  receiver.heartbeat(bsnType, bsnName, delay);
                }
                catch (RemoteException e) {
                  receiver = null;
                }
              }
              Thread.sleep(2000);
              checkVisible();
            }
            catch (InterruptedException e) {
            }
          }
          // System.out.println("hbeat: bye");
          hbthread = null;
        }

      };
      hbthread.start();
    }
  }

  public class BSNWindowReceiver extends UnicastRemoteObject implements BSNInterface {

    /**
     * @throws RemoteException
     */
    protected BSNWindowReceiver() throws RemoteException {
      super();
      // TODO Auto-generated constructor stub
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.ponder2.bsn.BSNInterface#setRate(java.lang.String)
     */
    public void setBsnRate(int value) throws RemoteException {
      BSN.this.setRate(value);
      rate.setValue(value / 1000.0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.ponder2.bsn.BSNInterface#getRate()
     */
    public int getBsnRate() throws RemoteException {
      //System.out.println("Got getRate()");
      int result = (int)(getRate().getValue() * 1000);
      //System.out.println("returning rate of " + result);
      return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.ponder2.bsn.BSNInterface#getValue()
     */
    public int getBsnValue() throws RemoteException {
      //System.out.println("Got getValue()");
      int result = (int)getValue().getValue();
      //System.out.println("returning value of " + result);
      return result;
    }
  }
} // @jve:decl-index=0:visual-constraint="10,10"
