/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Jul 5, 2006
 *
 * $Log:$
 */

package net.ponder2.bsn;

import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import net.ponder2.bsn.BSN.BSNType;

/**
 * TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class BSNController extends JFrame {

  private static final long serialVersionUID = 1L;
  private JPanel jContentPane = null;
  private JToggleButton bp = null;
  private JToggleButton heart = null;
  private JToggleButton glucose = null;
  private JToggleButton oxy = null;
  private JToggleButton Temp = null;
  private JToggleButton ipump = null;

  private Map<String, JFrame> bsns;
  private String patientName;

  /**
   * This is the default constructor
   */
  public BSNController(String patientName) {
    super();
    this.patientName = patientName;
    bsns = new HashMap<String, JFrame>();
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {
    this.setSize(250, 200);
    this.setContentPane(getJContentPane());
    this.setTitle(patientName + " BSN Controller");
    this.addWindowListener(new java.awt.event.WindowAdapter() {

      @Override
      public void windowClosing(java.awt.event.WindowEvent e) {
        System.exit(0);
      }
    });
  }

  /**
   * This method initializes jContentPane
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      jContentPane = new JPanel();
      jContentPane.setLayout(new BoxLayout(getJContentPane(), BoxLayout.Y_AXIS));
      jContentPane.add(getBp(), null);
      jContentPane.add(getHeart(), null);
      jContentPane.add(getGlucose(), null);
      jContentPane.add(getOxy(), null);
      jContentPane.add(getTemp(), null);
      jContentPane.add(getIpump(), null);
    }
    return jContentPane;
  }

  /**
   * This method initializes bp
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getBp() {
    if (bp == null) {
      bp = new JToggleButton();
      bp.setText("Blood Pressure");
      bp.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.BP, "BP1");
        }
      });
    }
    return bp;
  }

  /**
   * This method initializes heart
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getHeart() {
    if (heart == null) {
      heart = new JToggleButton();
      heart.setText("Heart");
      heart.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.HEART, "HEART1");
        }
      });
    }
    return heart;
  }

  /**
   * This method initializes glucose
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getGlucose() {
    if (glucose == null) {
      glucose = new JToggleButton();
      glucose.setText("Glucose");
      glucose.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.GLUCOSE, "GLUCOSE1");
        }
      });
    }
    return glucose;
  }

  /**
   * This method initializes oxy
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getOxy() {
    if (oxy == null) {
      oxy = new JToggleButton();
      oxy.setText("Oxygen");
      oxy.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.OXY, "OXY1");
        }
      });
    }
    return oxy;
  }

  /**
   * This method initializes Temp
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getTemp() {
    if (Temp == null) {
      Temp = new JToggleButton();
      Temp.setText("Temperature");
      Temp.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.TEMP, "TEMP1");
        }
      });
    }
    return Temp;
  }

  /**
   * This method initializes ipump
   * 
   * @return javax.swing.JToggleButton
   */
  private JToggleButton getIpump() {
    if (ipump == null) {
      ipump = new JToggleButton();
      ipump.setText("Insulin Pump");
      ipump.addActionListener(new java.awt.event.ActionListener() {

        public void actionPerformed(java.awt.event.ActionEvent e) {
          display((AbstractButton)e.getSource(), BSNType.IPUMP, "IPUMP1");
        }
      });
    }
    return ipump;
  }

  /**
   * @param b
   * @param bp2
   * @param string
   */
  protected void display(final AbstractButton button, BSNType type, final String name) {
    JFrame window = bsns.get(name);
    if (window == null) {
      if (type == BSNType.IPUMP) {
        InsulinPump ip = new InsulinPump();
        ip.init("pump", patientName, name);
        window = ip;
      }
      else {
        BSN bsn = new BSN();
        bsn.initialise(type, patientName, name);
        window = bsn;
      }
      bsns.put(name, window);
      window.addWindowListener(new java.awt.event.WindowAdapter() {

        @Override
        public void windowClosing(java.awt.event.WindowEvent e) {
          button.setSelected(false);
        }
      });
    }
    window.setVisible(button.isSelected());
  }

  public static void main(String[] args) {
    String name = "Demo";
    if (args.length > 0)
      name = args[0];
    BSNController bsnc = new BSNController(name);
    bsnc.setVisible(true);
  }

} // @jve:decl-index=0:visual-constraint="10,10"
