/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Jul 5, 2006
 *
 * $Log:$
 */

package net.ponder2.bsn;

import java.awt.FlowLayout;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Injects insulin into the patient with the specified dose
 * <p>
 * Part of the Ponder2 tutorial TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class InsulinPump extends JFrame {

  private static final long serialVersionUID = 1L;
  private JPanel jContentPane = null;
  private JLabel icon = null;

  private ImageIcon picture[];
  private BSNReceiverInterface receiver = null;
  boolean running = true;
  public final static String TYPE = "InsulinPump";
  private String name; // @jve:decl-index=0:
  private String patientName;

  /**
   * This is the default constructor
   */
  public InsulinPump() {
    super();
    picture = new ImageIcon[2];
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {
    this.setSize(118, 153);
    this.setContentPane(getJContentPane());
    this.setTitle("Insulin");
    this.addWindowListener(new java.awt.event.WindowAdapter() {

      @Override
      public void windowActivated(WindowEvent e) {
        // System.out.println("WindowListener method called: windowActivated.");
        if (InsulinPump.this.isVisible() && !running) {
          running = true;
          startHeartbeat();
        }
      }

      @Override
      public void windowClosing(java.awt.event.WindowEvent e) {
        // System.out.println("windowClosing()"); // TODO Auto-generated Event
        running = false;
      }
    });
    startHeartbeat();
  }

  public void init(String type, String patientName, String name) {
    this.patientName = patientName;
    this.name = name;
    initPicture(type);
    setVisible(true);
    try {
      Naming.rebind(patientName + "_" + name, new PumpReceiver());
    }
    catch (RemoteException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    catch (MalformedURLException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  public void initPicture(String name) {
    picture[1] = new ImageIcon(this.getClass().getResource("/resource/" + name + "On.jpg"));
    picture[0] = new ImageIcon(this.getClass().getResource("/resource/" + name + "Off.jpg"));
    icon.setIcon(picture[0]);
  }

  private void startPump(int amount) {
    icon.setIcon(picture[1]);
    new Thread() {

      @Override
      public void run() {
        try {
          sleep(2000);
          icon.setIcon(picture[0]);
        }
        catch (InterruptedException e) {
        }
      }
    }.start();
  }

  /**
   * This method initializes jContentPane
   * 
   * @return javax.swing.JPanel
   */
  private JPanel getJContentPane() {
    if (jContentPane == null) {
      icon = new JLabel();
      icon.setText("");
      jContentPane = new JPanel();
      jContentPane.setLayout(new FlowLayout());
      jContentPane.add(icon, null);
    }
    return jContentPane;
  }

  private void checkVisible() {
    if (!isVisible()) {
      running = false;
    }
  }

  private void startHeartbeat() {
    Thread hbthread = new Thread() {

      /*
       * (non-Javadoc)
       * 
       * @see java.lang.Thread#run()
       */
      @Override
      public void run() {
        while (running) {
          try {
            if (receiver == null) {
              try {
                // System.out.println("Pump trying to find BSNBus");
                receiver = (BSNReceiverInterface)Naming.lookup(patientName+"_"+"BSNBus");
              }
              catch (MalformedURLException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
              catch (RemoteException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
              catch (NotBoundException e) {
              }

            }
            if (receiver != null) {
              try {
                receiver.heartbeat(TYPE, name, 0);
              }
              catch (RemoteException e) {
                receiver = null;
              }
            }
            Thread.sleep(2000);
            checkVisible();
          }
          catch (InterruptedException e) {
          }
        }
        System.out.println(name + " stopping");
      }

    };
    hbthread.start();
  }

  public class PumpReceiver extends UnicastRemoteObject implements PumpInterface {

    /**
     * @throws RemoteException
     */
    protected PumpReceiver() throws RemoteException {
      super();
      // TODO Auto-generated constructor stub
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.ponder2.bsn.PumpInterface#inject(java.lang.String)
     */
    public void inject(int amount) throws RemoteException {
      startPump(amount);
    }
  }
} // @jve:decl-index=0:visual-constraint="10,10"
