package net.ponder2.bsn;

import java.awt.BorderLayout;
import java.util.Dictionary;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;

/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Jul 2, 2006
 *
 * $Log:$
 */

/**
 * TODO Description
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class SliderValue extends JPanel {

  private static final long serialVersionUID = 1L;
  private JSlider slider = null;
  private JLabel value = null;

  String title; // @jve:decl-index=0:
  Dictionary<Integer, JLabel> labelTable; // @jve:decl-index=0:

  private int min, max;

  private JLabel titleLabel = null;
  private String units = "";

  /**
   * This is the default constructor
   */
  public SliderValue() {
    super();
    labelTable = null;
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {
    titleLabel = new JLabel();
    titleLabel.setText("JLabel");
    titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    this.setSize(300, 200);
    this.setLayout(new BorderLayout());
    this.add(getSlider(), BorderLayout.WEST);
    this.add(getValueLabel(), BorderLayout.CENTER);
    this.add(titleLabel, BorderLayout.NORTH);
  }

  /**
   * This method initializes slider
   * 
   * @return javax.swing.JSlider
   */
  private JSlider getSlider() {
    if (slider == null) {
      slider = new JSlider();
      slider.addChangeListener(new javax.swing.event.ChangeListener() {

        public void stateChanged(javax.swing.event.ChangeEvent e) {
          if (slider.getValueIsAdjusting())
            return;
          String oldValue = "00";
          // System.out.println("stateChanged()");
          int newValue = slider.getValue();
          String svalue = "" + newValue;
          if (labelTable != null) {
            svalue = labelTable.get(newValue).getText();
          }
          if (value != null) {
            oldValue = value.getText();
            value.setText(svalue + " " + units);
          }

          sendValue(oldValue, svalue);
        }
      });
      slider.setMajorTickSpacing(1);
      // slider.setMinorTickSpacing(1);
      slider.setMinimum(0);
      slider.setMaximum(5);
      slider.setValue(2);
      slider.setSnapToTicks(true);
      slider.setPaintTicks(true);
      // slider.setPaintTrack(true);
      slider.setEnabled(true);
    }
    return slider;
  }

  /**
   * This method initializes value
   * 
   * @return javax.swing.JLabel
   */
  private JLabel getValueLabel() {
    if (value == null) {
      value = new JLabel();
      value.setText("");
    }
    return value;
  }

  private void sendValue(String ovalue, String nvalue) {
    this.firePropertyChange("value", ovalue, nvalue);
  }

  /**
   * @param min
   *          the min to set
   */
  public void setMinMax(int min, int max) {
    this.min = min;
    this.max = max;
    setRange();
  }

  /**
   * @param title
   *          the title to set
   */
  public void setTitle(String title) {
    titleLabel.setText(title);
  }

  public void setUnits(String units) {
    this.units = units;
  }

  /**
   * @param value
   *          the value to set
   */
  public void setValue(double newValue) {
    // System.out.println("Value set to " + newValue);
    int value = 0;
    if (labelTable == null) {
      value = (int)(newValue + 0.4);
    }
    else {
      int i;
      for (i = 0; i < labelTable.size(); i++) {
        String label = labelTable.get(i).getText();
        double l = Double.parseDouble(label);
        if (newValue <= (l + 0.05))
          break;
      }
      value = i;
    }
    slider.setValue(value);
  }

  public double getValue() {
    int value = slider.getValue();
    // System.out.println("SliderValue: getValue: Slider value is " + value);
    if (labelTable == null)
      return value;
    // System.out.println("SliderValue: getValue: Returning labelTable("+value+
    // ") "+labelTable.get(value).getText());
    return Double.parseDouble(labelTable.get(value).getText());
  }

  /**
   * @param labelTable
   *          the labelTable to set
   */
  public void setLabelTable(Dictionary<Integer, JLabel> labelTable) {
    this.labelTable = labelTable;
    setMinMax(0, labelTable.size() - 1);
    slider.setLabelTable(labelTable);
    slider.setPaintLabels(true);
  }

  private void setRange() {
    // Set major tick marks every 5 units
    int span = max - min;
    slider.setMaximum(max);
    slider.setMinimum(min);
    if (span > 40) {
      slider.setMajorTickSpacing(20);
      slider.setMinorTickSpacing(10);
    }
    else if (span > 20)
      slider.setMajorTickSpacing(10);
    else if (span > 10)
      slider.setMajorTickSpacing(5);
    else
      slider.setMajorTickSpacing(1);
  }

}
