/**
 * Created on Sep 14, 2005
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 */

package net.ponder2.managedobject;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.tutorial.AlarmWindow;

/**
 * Displays an alarm clock in its own window. The alarm clock can be shown and
 * hidden, it can also be set to ring or to stop ringing.
 * <p>
 * Part of the Ponder2 Tutorial
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class AlarmDisplay implements ManagedObject {

  AlarmWindow alarmWindow;
  String title = null;

  /**
   * Creates an instance of the Alarm Clock with the title aString.
   * 
   * @param title
   */
  @Ponder2op("create:")
  public AlarmDisplay(String aString) {
    alarmWindow = new AlarmWindow();
    title = aString;
    alarmWindow.setTitle(title);
    alarmWindow.setLocationByPlatform(true);
  }

  /**
   * Sets the alarm ringing if aBoolean is true otherwise silences it
   * 
   * @param aBoolean
   */
  @Ponder2op("setAlarm:")
  public void setAlarm(boolean aBoolean) {
    System.out.println("AlarmDisplay: alarm is set to " + aBoolean);
    alarmWindow.setAlarm(aBoolean);
  }

  /**
   * Sets the alarm ringing for a duration of anInt seconds
   * 
   * @param anInt
   */
  @Ponder2op("setShortAlarm:")
  public void setShortAlarm(int anInt) {
    System.out.println("AlarmDisplay: short alarm is set to " + anInt + " second(s)");
    alarmWindow.setAlarm(true);
    startTimer(anInt);
  }

  /**
   * Shows the Alarm Clock window on the screen
   * 
   */
  @Ponder2op("show")
  public void show() {
    alarmWindow.setVisible(true);
  }

  /**
   * Hides the Alarm Clock window
   * 
   */
  @Ponder2op("hide")
  public void hide() {
    alarmWindow.setVisible(false);
  }

  private void startTimer(final int seconds) {
    Thread thread = new Thread() {

      @Override
      public void run() {
        try {
          sleep(seconds * 1000);
        }
        catch (InterruptedException e) {
        }
        alarmWindow.setAlarm(false);
      }
    };
    thread.start();
  }

}
