/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Nov 15, 2005
 *
 * $Log: BSNAdaptor.java,v $
 * Revision 1.5  2006/03/15 14:40:46  kpt
 * Kept wrong version, stepped back
 *
 * Revision 1.3  2005/11/23 14:47:07  kpt
 * Removed Iterable from QDParser.
 *
 * Revision 1.2  2005/11/17 11:36:28  kpt
 * Added context to many ops for arg substitution
 *
 * Revision 1.1  2005/11/15 16:33:25  kpt
 * Result added to create
 *
 */

package net.ponder2.managedobject;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.bsn.BSNInterface;

/**
 * An adaptor to talk to BSN nodes
 * <p>
 * Part of the Ponder2 Tutorial
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class BSNAdaptor implements ManagedObject {

  String name;
  BSNInterface bsn;

  /**
   * Creates an instance of the adaptor with name aName belonging to
   * aPatient. The RMI name used will be "aPatient_aName"
   * 
   * @param aName
   * @param aPatient
   */
  @Ponder2op("create:patient:")
  public BSNAdaptor(String aName, String aPatient) {
    name = aPatient + "_" + aName;
    getBSN();
  }

  /**
   * Sets the sampling rate of the BSN node to anInteger. anInteger is expressed
   * in milliseconds.
   * 
   * @param anInteger
   */
  @Ponder2op("setSampleRate:")
  public void setRate(int anInteger) {
    try {
      if (getBSN())

        bsn.setBsnRate(anInteger);
    }
    catch (RemoteException e) {
      // We can't seem to contact the BSN
      bsn = null;
    }
  }

  /**
   * Interrogates the node over the RMI link and answers with the rate in
   * millseconds
   * 
   * @return the rate in milliseconds
   */
  @Ponder2op("getSampleRate")
  public int getRate() {
    int value = 0;
    try {
      if (getBSN())
        value = bsn.getBsnRate();
      //System.out.println("Just got a rate of " + value);
    }
    catch (RemoteException e) {
      // We can't seem to contact the BSN
      bsn = null;
    }
    return value;
  }

  /**
   * Interrogates the node over the RMI link and answers with the BSN reading
   * corresponding to its type
   * 
   * @return
   */
  @Ponder2op("getValue")
  public int getValue() {
    int value = 0;
    try {
      if (getBSN())
        value = bsn.getBsnValue();
    }
    catch (RemoteException e) {
      // We can't seem to contact the BSN
      bsn = null;
    }
    return value;
  }

  private boolean getBSN() {
    if (name != null && bsn == null)
      try {
        System.out.println("BSNAdaptor looking up " + name);
        bsn = (BSNInterface)Naming.lookup(name);
      }
      catch (MalformedURLException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      catch (RemoteException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      catch (NotBoundException e) {
        System.out.println("Cannot find my node " + name);
      }
    return bsn != null;
  }
}
