/**
 * Copyright  2005 Imperial College, London, England.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Contact: Kevin Twidle <kpt@doc.ic.ac.uk>
 *
 * Created on Nov 15, 2005
 *
 * $Log: BSNAdaptor.java,v $
 * Revision 1.5  2006/03/15 14:40:46  kpt
 * Kept wrong version, stepped back
 *
 * Revision 1.3  2005/11/23 14:47:07  kpt
 * Removed Iterable from QDParser.
 *
 * Revision 1.2  2005/11/17 11:36:28  kpt
 * Added context to many ops for arg substitution
 *
 * Revision 1.1  2005/11/15 16:33:25  kpt
 * Result added to create
 *
 */

package net.ponder2.managedobject;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import net.ponder2.ManagedObject;
import net.ponder2.apt.Ponder2op;
import net.ponder2.bsn.PumpInterface;

/**
 * Injects Insulin into the patient with the specified dose. This managed object
 * acts as an interface to the real pump.
 * <p>
 * Part of the Ponder2 tutorial
 * 
 * @author Kevin Twidle
 * @version $Id:$
 */
public class PumpAdaptor implements ManagedObject {

  String name;
  PumpInterface pump;

  /**
   * Creates an instance of the adaptor with name aName belonging to
   * aPatient. The RMI name used will be "aPatient_aName"
   * 
   * @param aName
   * @param aPatient
   */
  @Ponder2op("create:patient:")
  public PumpAdaptor(String aName, String aPatient) {
    name = aPatient + "_" + aName;
    getPump();
  }

  /**
   * injects aDose (integer) ml of Insulin into the patient
   * 
   * @param dose
   */
  @Ponder2op("inject:")
  public void inject(int aDose) {
    try {
      if (getPump())
        pump.inject(aDose);
    }
    catch (RemoteException e) {
      pump = null;
    }

  }

  private boolean getPump() {
    if (name != null && pump == null)
      try {
        pump = (PumpInterface)Naming.lookup(name);
      }
      catch (MalformedURLException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      catch (RemoteException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      catch (NotBoundException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    return pump != null;
  }

}
